-- Semi Auto Recast START

--[[
	Sarf's SemiAutoRecast script v0.4 - 2004-06-29 20:33 GMT+2
	Based on code by wonderful. 
	This is simply a refactoring of wonderful's code to make it easier to extend.

	wonderful's original post : http://www.worldofwarcraft.com/thread.aspx?FN=wow-interface-customization&T=3383
]]--

-- If set to false, will disable SemiAutoRecast completely.
SemiAutoRecast_ModEnabled = false;

SemiAutoRecast_Address = nil;
SemiAutoRecast_Length = 0;

AUTOKEY = 0;
CURKEY = 1;

-- If set to 1, it will enable recasting while in combat.
SemiAutoRecast_AllowRecastInCombat = false;

-- If set to 0, it will disable recasting while out of combat.
SemiAutoRecast_AllowRecastOutOfCombat = true;

SemiAutoRecast_AutoRecastList = { };
SemiAutoRecast_AutoRecastList[1] = { barnumber = 6, buttonnumber = 11, duration = 150, use_wait = 2, time_recast = 0 };
SemiAutoRecast_AutoRecastList[2] = { barnumber = 6, buttonnumber = 12, duration = 150, use_wait = 2, time_recast = 0 };

--[[
SemiAutoRecast_AutoRecastList contains something I call AutoRecastElements.

Each AutoRecastElements has these properties :

        required:

        barnumber - which actionbar the actionbutton is located on
        buttonnumber - which actionbutton to press
        duration - how long to wait before firing off the recast again

        
        optional:

        target - what target to switch to prior to firing off the recast 
                 (will try to switch back to last target)
                 also, see specialtargetlist
        use_wait - the minimum time to wait before firing off another 
                   auto recast (defaults to 2)
        allow_in_combat - if set to true it will allow this recast to happen 
                          in combat 
                          (it defaults to SemiAutoRecast_AllowRecastInCombat)
        allow_out_of_combat - if set to true it will allow this recast out 
                              of combat 
                              (it defaults to SemiAutoRecast_AllowRecastOutOfCombat)


        managed: - these properties should not be meddled with unnecessarily

        time_recast - placeholder variable that contains the next time 
                      period to wait 'til (zero to fire ASAP)
        
        
        reserved: - for future expansion (not implemented yet)
        
        target_buff_tooltip - will recast if the buff_tooltip does not match 
                              any of the buffs present on the unit currently 
                              targeted



        specialtargetlist:
        
        You can use certain strings as target to make the targeting system 
        act in a certain way.
        
        combatcastplayer - will target yourself, fire off, then switch back 
                           to the last enemy
        combatcastpartyX - will target party member X, fire off, then switch 
                           back to the last enemy
        
        Note that the combatcast* target strings aren't particularly useful 
        (except for ranged casters) if recasts are not allowed in combat 
        (i.e. if SemiAutoRecast_AllowRecastInCombat is set to zero, or allow_in_combat is set to 1).
        
]]--

-- Since I do not know how a sure-fire way to find out how many "rows" a 
-- table has this variable should to be updated whenever the 
-- SemiAutoRecast_AutoRecastList is.
-- I've found that a certain function/method can be used - table.getn() - 
-- but it has certain... interesting... properties.
-- Change SemiAutoRecast_GetNumberOfRecasts() to start using it.
-- NUMBEROFKEY = 2;

-- Returns the number of recasts in the list.
function SemiAutoRecast_GetNumberOfRecasts()
        -- return NUMBEROFKEY;
        return getn(SemiAutoRecast_AutoRecastList);
end

-- Should return the current targets unit or nil if no target.
function SemiAutoRecast_GetTargetUnit()
        return nil;
        --[[
        local targetFrame = GetGlobal("TargetFrame");
        if ( targetFrame ~= nil ) then
                return targetFrame.unit;
        end
        return nil;
        ]]--
end

-- Handles a target according to what the target is.
-- If exiting is set to false means that it is BEFORE firing a recast
-- IF exiting is set to true means that it is AFTER firing a recast
function SemiAutoRecast_HandleTarget(target, exiting)
        if ( strsub(target, 1, 10) == "combatcast" ) then
                if ( exiting == false ) then
                        TargetUnit(strsub(target, 11));
                else
                        TargetLastEnemy();
                end
        else
                return nil;
        end
        return 1;
end

-- Checks if the currently selected target has a buff that matches the specified tooltip
-- Returns the tooltip if found, otherwise nil.
-- CURRENTLY NOT WORKING
function SemiAutoRecast_TargetHasBuffTooltip(target_buff_tooltip)
        return nil;
end

-- Callback method for the Cosmos settings
function SemiAutoRecast_HandleEnableChange(checked)
        if (checked == 1) then
                DEBUG_MSG("SemiAutoRecast activated", 1);
                SemiAutoRecast_ModEnabled = true;
        else
                SemiAutoRecast_ModEnabled = false;
                DEBUG_MSG("SemiAutoRecast deactivated", 1);
        end
end

function SemiAutoRecast_OnLoad()
        if (CosmosMaster_Register ~= nil) then
                CosmosMaster_Register("COS_SAR_SECTION",
                        "SEPARATOR",
                        "Semi Auto Recast",
                        "This section contains the settings for the semi-automatic recast."
                );
                CosmosMaster_Register("COS_SAR_ENABLE",
                        "CHECKBOX",
                        "Turn on Semi Auto Recast",
                        "Recast/reuse abilities when you press movement buttons",
                        SemiAutoRecast_HandleEnableChange,
                        1,
                        1,
                        0,
                        1,
                        "",
                        .01,
                        1,
                        "\%"
                );
                SemiAutoRecast_Address, SemiAutoRecast_Length = CosmosMaster_RegisterCVar("COS_SAR_RECASTLIST", "", CSM_STRING, 0, 200);
        else
                SemiAutoRecast_HandleEnableChange(1);
        end
end

-- Performs one auto recast action if needed, then waits a while.
function SemiAutoFind()
        if ( SemiAutoRecast_ModEnabled == false ) then
                return;
        end
        local time = GetTime();
        local recast = SemiAutoRecast_AutoRecastList[CURKEY];
        if ( (IN_ATTACK_MODE == false) ) then
                if ( (recast.allow_out_of_combat == false) and (recast.allow_out_of_combat ~= nil) ) then
                        return;
                end
                if ( (SemiAutoRecast_AllowRecastOutOfCombat == false) ) then
                        return;
                end
        end
        if ( (IN_ATTACK_MODE == true) ) then
                if ( (recast.allow_in_combat == false) or (recast.allow_in_combat == nil) ) then
                        return;
                end
                if ( (SemiAutoRecast_AllowRecastInCombat == false) ) then
                        return;
                end
        end
        if (time > AUTOKEY ) then
                local temp_recast_use_wait = nil;
                local previousTarget = nil;
                if(time > recast.time_recast)
                        local curBar = CURRENT_ACTIONBAR_PAGE;
                        CURRENT_ACTIONBAR_PAGE = recast.barnumber;
                        ChangeActionBarPage();
                        
                        if (recast.target ~= nil) then
                                previousTarget = SemiAutoRecast_GetTargetUnit();
                                if ( SemiAutoRecast_HandleTarget(recast.target, false) ~= nil) then 
                                        TargetUnit(recast.target);
                                end
                        end
                        
                        if ( (recast.target_buff_tooltip == nil) or (SemiAutoRecast_TargetHasBuffTooltip(recast.target_buff_tooltip) == nil) ) then
                                ActionButtonDown(recast.buttonnumber);
                                ActionButtonUp(recast.buttonnumber);
                                CURRENT_ACTIONBAR_PAGE = curBar;
                                ChangeActionBarPage();

                                recast.time_recast = time + recast.duration;

                                temp_recast_use_wait = recast.use_wait;

                                if ( temp_recast_use_wait == nil ) then
                                        temp_recast_use_wait = 2;
                                end
                        else
                                -- the ability in question was already present - wait for a little while then go on
                                temp_recast_use_wait = 1;
                        end
        
                        if (recast.target ~= nil) then
                                if ( SemiAutoRecast_HandleTarget(recast.target, true) ~= nil) then 
                                        if ( previousTarget ~= nil ) then 
                                                TargetUnit(previousTarget);
                                        end
                                end
                        end
                        
                        AUTOKEY = time + temp_recast_use_wait;
                else
                        -- it was not time for us to recast the ability in question - wait for a while
                        AUTOKEY = time + 1;
                end
                CURKEY = CURKEY + 1;
                if (CURKEY > SemiAutoRecast_GetNumberOfRecasts()) then
                        CURKEY = 1;
                end
        end
end

-- Semi Auto Recast END
