-- Neat-o functions for dealing with recasts follow

-- Finds the recast and returns its index, or nil if it could not find the recast in the recast list.
function SemiAutoRecast_FindRecastIndex(recast)
        -- we assume that the user sent us some instructions referencing the recast
        if( (recast.barnumber ~= nil) and (recast.buttonnumber ~= nil) ) then
                for index, elementRecast in SemiAutoRecast_AutoRecastList do
                        if ( (elementRecast.barnumber == recast.barnumber) and (elementRecast.buttonnumber == recast.buttonnumber) ) then
                                return index;
                        end
                end
        end
        return nil;
end

-- Adds the recast to the list, if the recast is correct AND if there is no 
-- recast with the same bar/button combination already specified in the list.
-- Returns the recast if succesful, otherwise nil.
function SemiAutoRecast_AddRecast(recast)
        if( (recast.barnumber ~= nil) and (recast.buttonnumber ~= nil) and (recast.duration ~= nil) ) then
                local index = SemiAutoRecast_FindRecastIndex(recast);
                if (index == nil) then
                        table.insert(SemiAutoRecast_AutoRecastList, recast);
                        return recast;
                end
        end
        return nil;
end

-- Copies all managed variables (variables used and maintained by SemiAutoRecast, that is) to the destination recast.
function SemiAutoRecast_CopyAllManagedVariables(source, destination)
        destination.time_recast = source.time_recast;
end


-- Updates the recast in the list, if the recast is correct AND if there is a 
-- recast with the same bar/button combination already specified in the list.
-- Returns the recast if succesful, otherwise nil.
function SemiAutoRecast_UpdateRecast(recast)
        if( (recast.barnumber ~= nil) and (recast.buttonnumber ~= nil) and (recast.duration ~= nil) ) then
                local index = SemiAutoRecast_FindRecastIndex(recast);
                if (index ~= nil) then
                        SemiAutoRecast_CopyAllManagedVariables(SemiAutoRecast_AutoRecastList[index], recast);
                        SemiAutoRecast_AutoRecastList[index] = recast;
                        return recast;
                end
        end
        return nil;
end

-- Removes the recast from the list, if the recast is either a number or 
-- if there is a recast with the bar/button combination specified in the list.
-- Returns the removed recast if succesful, otherwise nil.
function SemiAutoRecast_RemoveRecast(recast)
        if ( type(recast) == "number" ) then
                return table.remove(SemiAutoRecast_AutoRecastList, recast);
        else
                local index = SemiAutoRecast_GetRecastIndex(recast);
                if (index ~= nil) then
                        return table.remove();
                end
                return nil;
        end
end


-- Neat-o functions for dealing with recasts end

-- SemiAutoRecast Compression/Decompression functions start

function SemiAutoRecast_CompressBoolean(bool)
				if (bool == nil) then
								return "";
        end
        if (bool == true) then
                return "t";
        else
                return "f";
        end
end

-- idea : use "extended" decimal representation (using [0..9,A..Z,a..z] to store numbers) - would this be space efficient
function SemiAutoRecast_CompressInteger(integer)
				if (integer == nil) then
								return "";
        end
        return string.format("%i", (integer + 0));
end

function SemiAutoRecast_CompressTargetString(str)
				if(string.find(str, "combatcast") ~= nil) then
								if(str == "combatcastplayer") then
												return "%p";
								end
								if(string.find(str, "combatcastparty") ~= nil) then
												return "%"..string.sub(str, string.len(str)-1);
								end
				end
				return str;
end

function SemiAutoRecast_CompressString(str)
				if (str == nil) then
								return "";
				else
								return SemiAutoRecast_CompressTargetString(str);
        end
end

function SemiAutoRecast_DecompressBoolean(str)
				if (str == nil) then
								return nil;
        end
        if ( (bool == "t") or (bool == "true") ) then
                return true;
        else
                return false;
        end
end

function SemiAutoRecast_DecompressInteger(str)
				if (integer == nil) then
								return nil;
        end
        return (str + 0);
end

function SemiAutoRecast_DeCompressTargetString(str)
				if(string.find(str, "%") ~= nil) then
								if(str == "%p") then
												return "combatcastplayer";
								end
								if(string.find(str, "%") ~= nil) then
												return "combatcastparty"..string.sub(str, string.len(str)-1);
								end
				end
				return str;
end

function SemiAutoRecast_DecompressString(str)
				if (str == nil) then
								return nil;
				else
								return str;
        end
end

function SemiAutoRecast_CompressRecast(recast)
        local newStr = "";
        return string.format("%i,%i,%i,%s,%i,%s,%s", 
          SemiAutoRecast_CompressInteger(recast.barnumber), 
          SemiAutoRecast_CompressInteger(recast.buttonumber), 
          SemiAutoRecast_CompressInteger(recast.duration),
          SemiAutoRecast_CompressString(recast.target),
          SemiAutoRecast_CompressInteger(recast.use_wait),
          SemiAutoRecast_CompressBoolean(recast.allow_in_combat),
          SemiAutoRecast_CompressBoolean(recast.allow_out_of_combat)
        );
end

function SemiAutoRecast_GetNextIndex(index, str, pattern)
        local newIndex = string.find(str, ",", index);
        if(newIndex == nil) then
                newIndex = string.len(str);
        else
                newIndex = newIndex + 1;
        end
        return newIndex;
end

function SemiAutoRecast_DecompressRecast(recast)
        local index = 1;
        local prevIndex = 1;
        local newRecast = {};
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.barnumber = SemiAutoRecast_DecompressInteger(string.sub(recast, prevIndex, index));
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.buttonumber = SemiAutoRecast_DecompressInteger(string.sub(recast, prevIndex, index));
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.duration = SemiAutoRecast_DecompressInteger(string.sub(recast, prevIndex, index));
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.duration = SemiAutoRecast_DecompressString(string.sub(recast, prevIndex, index));
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.use_wait = SemiAutoRecast_DecompressInteger(string.sub(recast, prevIndex, index));
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.allow_in_combat = SemiAutoRecast_DecompressBoolean(string.sub(recast, prevIndex, index));
        prevIndex = index;
        index = SemiAutoRecast_GetNextIndex(prevIndex, recast, ","); 
        newRecast.allow_out_of_combat = SemiAutoRecast_DecompressBoolean(string.sub(recast, prevIndex, index));
        return newRecast;
end

-- SemiAutoRecast Compression/Decompression functions end

-- SemiAutoRecast Save/Load CVar functions start

function SemiAutoRecast_GetRecastListAsString()
        local str = "";
        for index, element in SemiAutoRecast_AutoRecastList do
                str = str..SemiAutoRecast_CompressRecast(element)..";";
        end
        return str;
end

function SemiAutoRecast_GetRecastStringAsList(str)
        local list = {};
        for w in string.gfind(s, ";") do
                if(string.len(w) > 0) then
                        table.insert(list, SemiAutoRecast_DeompressRecast(w));
                end
        end
        return list;
end

function SemiAutoRecast_LoadRecastList(cvar)
        if ( SemiAutoRecast_Address ~= nil ) then
                local str = CosmosMaster_GetCVar(cvar);
                if (string.len(str) > 0 ) then
                        SemiAutoRecast_AutoRecastList = SemiAutoRecast_GetRecastListAsString(str);
                end
        end
end

function SemiAutoRecast_SaveRecastList(cvar)
        if ( SemiAutoRecast_Address ~= nil ) then
                local str = SemiAutoRecast_GetRecastListAsString();
                CosmosMaster_SetCVar(cvar, list);
        end
end
-- SemiAutoRecast Save/Load CVar functions end